
# q-expansion of newform 2160.2.q.g, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2160
weight = 2
poly_data = [4, 0, -2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 2, 0, 1], 2], [[0, 4, 0, -1], 2]]

hecke_ring_character_values = [[271, [1, 0, 0, 0]], [1621, [1, 0, 0, 0]], [2081, [-1, 1, 0, 0]], [1297, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [-1, 1, 0, 0], [0, 1, -1, 0], [0, 0, 0, 0], [0, 0, 0, 0], [2, 0, 0, 0], [2, 0, 0, -2], [5, -5, 1, -1], [0, 3, 2, 0], [6, -6, -2, 2], [-6, 0, 0, 0], [-5, 5, -2, 2], [0, 2, -4, 0], [0, 7, -1, 0], [2, 0, 0, 4], [6, -6, -2, 2], [0, 3, 0, 0], [-1, 1, -5, 5], [0, 0, 0, -4], [4, 0, 0, -4], [0, -2, 2, 0], [0, 3, -1, 0], [7, 0, 0, 4], [0, -2, 0, 0], [0, -2, 0, 0], [10, -10, 0, 0], [-5, 0, 0, 3], [-1, 0, 0, 4], [0, 0, -4, 4], [7, 0, 0, 3], [2, -2, -2, 2], [0, 0, -8, 0], [0, 0, 8, -8], [-21, 21, 0, 0], [0, 12, 0, 0], [14, -14, 4, -4], [-2, 0, 0, -4], [-1, 1, -3, 3], [0, -12, 0, 0], [-4, 0, 0, 4], [5, 0, 0, 6], [0, 10, -2, 0], [12, -12, -4, 4], [-4, 0, 0, -8], [-18, 0, 0, 2], [12, -12, 0, 0], [0, 3, -1, 0], [0, 18, 4, 0], [9, -9, -2, 2], [10, 0, 0, -4], [-12, 12, 4, -4], [0, -11, -6, 0], [-2, 0, 0, -2], [18, -18, 4, -4], [0, -18, 0, 0], [-3, 0, 0, 8], [-26, 0, 0, 2], [0, 14, 0, 0], [0, -13, -2, 0], [-11, 11, 3, -3], [-2, 2, 8, -8], [17, 0, 0, 5], [2, -2, 2, -2], [0, -4, -8, 0], [0, -16, 4, 0], [0, -2, 2, 0], [20, -20, 4, -4], [6, -6, 8, -8], [0, 17, 6, 0], [0, 0, 0, 0], [14, 0, 0, -6], [0, 22, 0, 0], [-2, 2, -8, 8], [-22, 0, 0, 2], [-2, 2, 0, 0], [0, 3, 8, 0], [-22, 0, 0, 0], [6, -6, 8, -8], [-10, 10, 8, -8], [-20, 20, 0, 0], [0, -6, 8, 0], [10, 0, 0, -2], [-2, 0, 0, 4], [0, -8, 4, 0], [0, -9, -7, 0], [-2, 0, 0, 8], [0, 2, 8, 0], [0, 17, 2, 0], [22, -22, -4, 4], [-2, 0, 0, -4], [0, -6, 10, 0], [-6, 0, 0, -8], [4, -4, 4, -4], [-28, 28, 0, 0], [3, 0, 0, -9], [-9, 9, 2, -2], [-19, 0, 0, -4], [27, 0, 0, -3], [17, 0, 0, 2], [0, -29, -5, 0], [-34, 0, 0, 4], [3, -3, 3, -3], [0, 10, 0, 0], [16, -16, 4, -4], [-26, 0, 0, 0], [0, -15, -3, 0], [2, 0, 0, -12], [10, -10, 6, -6], [0, -14, 0, 0], [-15, 15, 7, -7], [-10, 0, 0, 8], [4, -4, -16, 16], [0, -10, 6, 0], [-4, 0, 0, -8], [0, -9, 2, 0], [-3, 3, -7, 7], [9, 0, 0, 1], [18, -18, 4, -4], [0, -20, -4, 0], [-10, 10, 0, 0], [0, 16, 0, 0], [0, 24, 4, 0], [-6, 0, 0, -12], [0, 34, -2, 0], [-1, 0, 0, 12], [0, 5, 6, 0], [6, 0, 0, 14], [0, -19, 5, 0], [10, -10, 8, -8], [38, 0, 0, 2], [-15, 15, 9, -9], [36, -36, -4, 4], [12, 0, 0, -8], [35, -35, 0, 0], [15, -15, -4, 4], [-16, 0, 0, -4], [-30, 30, -8, 8], [-8, 8, 0, 0], [30, 0, 0, 0], [-18, 0, 0, -2], [0, -7, -4, 0], [-1, 1, -3, 3], [7, 0, 0, -19], [11, 0, 0, -16], [0, 10, 6, 0], [0, -28, -4, 0], [0, 20, 8, 0], [0, 0, 16, -16], [55, 0, 0, -1], [20, -20, -8, 8], [-11, 0, 0, -2], [23, 0, 0, 9], [18, -18, -8, 8], [0, -29, -3, 0], [0, 14, 10, 0], [-10, 0, 0, 6], [0, 18, -8, 0], [8, 0, 0, 20], [13, -13, -6, 6], [0, 29, 7, 0], [24, 0, 0, 4], [17, -17, 13, -13], [46, 0, 0, 6], [16, -16, -12, 12], [0, 9, -1, 0], [-14, 0, 0, 6], [0, -6, -12, 0], [-19, 0, 0, 12], [-6, 6, 4, -4], [0, -28, 12, 0], [-29, 29, -6, 6], [28, -28, 0, 0], [0, 8, 0, 0], [-28, 28, -12, 12], [6, -6, 16, -16], [0, -36, -8, 0], [30, 0, 0, 8], [-3, 0, 0, -1], [0, -35, -12, 0], [0, -2, 0, 0], [0, -2, 10, 0], [38, -38, -8, 8], [-24, 0, 0, 12], [-39, 39, -1, 1], [0, -49, 2, 0], [20, 0, 0, -8], [0, -7, -3, 0], [11, -11, 2, -2], [6, 0, 0, 14], [4, 0, 0, -8], [0, 15, 1, 0], [26, 0, 0, -2], [0, 21, -12, 0], [-1, 0, 0, 5], [20, -20, 0, 0], [-7, 7, -2, 2], [0, -16, 4, 0], [0, 30, 16, 0], [-5, 0, 0, 1], [2, -2, -8, 8], [0, 20, 4, 0], [32, -32, 0, 0], [0, 19, -4, 0], [-4, 0, 0, 0], [-12, 0, 0, 4], [16, 0, 0, 16], [-13, 13, -7, 7], [0, -27, -16, 0], [-20, 20, -8, 8], [26, 0, 0, 16], [-10, 10, -8, 8], [0, -41, 7, 0], [0, -14, -4, 0], [-20, 20, -8, 8], [0, -31, 6, 0], [25, -25, -3, 3], [0, -26, 16, 0], [-11, 0, 0, -23], [-36, 36, -12, 12], [-5, 5, 4, -4], [32, -32, -12, 12], [-51, 51, -8, 8], [37, 0, 0, 7], [29, -29, 9, -9], [0, -22, 0, 0], [0, -6, 28, 0], [-12, 0, 0, -4], [0, -11, 9, 0], [0, -6, -26, 0], [-50, 50, 4, -4], [16, 0, 0, 8], [8, -8, -4, 4], [-41, 41, 10, -10], [0, -3, -1, 0], [0, 15, -3, 0], [-1, 1, 10, -10], [48, 0, 0, -4], [2, -2, 18, -18], [-20, 0, 0, -8], [0, 15, 11, 0], [4, 0, 0, -4], [-21, 21, 13, -13], [-1, 0, 0, -14], [-50, 50, -4, 4], [0, -6, -18, 0], [-1, 0, 0, 13], [-42, 42, -14, 14], [-30, 30, -10, 10], [-13, 0, 0, -3], [-22, 22, 0, 0], [-39, 0, 0, 4], [26, -26, 16, -16], [0, 7, 22, 0], [0, 4, -4, 0], [0, 0, 0, -8], [-14, 0, 0, 24], [0, -17, 5, 0], [-18, 0, 0, 0], [-10, 0, 0, 8], [0, 43, -5, 0], [0, 1, -9, 0], [14, -14, 16, -16], [2, 0, 0, 24], [-28, 28, 8, -8], [0, -14, -18, 0], [-26, 0, 0, -8], [0, -41, 8, 0], [-47, 47, 9, -9], [14, -14, 0, 0], [10, -10, 8, -8], [-15, 0, 0, -7], [0, -10, -16, 0], [46, -46, 6, -6], [-60, 60, -8, 8], [-27, 0, 0, -19], [45, -45, -3, 3], [0, 6, -16, 0], [-21, 0, 0, 14], [0, 0, 28, 0], [-18, 18, -16, 16], [40, -40, 16, -16], [0, 2, 16, 0], [0, 11, 14, 0], [7, -7, 5, -5], [-60, 0, 0, -8], [-48, 0, 0, 12], [20, -20, 8, -8], [0, 52, 4, 0], [51, 0, 0, 2], [0, -2, 16, 0], [-30, 0, 0, 8], [-42, 42, 16, -16], [-10, 10, 6, -6], [0, -10, -12, 0], [11, -11, 24, -24], [0, -32, -4, 0], [0, 0, 12, 0], [-18, 0, 0, -18], [0, 50, 0, 0], [-46, 46, -4, 4], [4, 0, 0, -28], [-22, 0, 0, -18]]
